	package ru.amse.smyshlyaev.grapheditor.graphdatacalculator;

	import java.awt.Graphics;

	import ru.amse.smyshlyaev.grapheditor.graph.*;


	public class EdgeDataCalculator {
		public static boolean isInEdge(Graphics gc, IEdge edge, 
		                 		int x, int y) {
			final int SHIFT = 3;
			int endX = getEndX(gc, edge) - getStartX(gc, edge);
			int endY = getEndY(gc, edge) - getStartY(gc, edge);
			int pointX = x - getStartX(gc, edge);
			int pointY = y - getStartY(gc, edge);
			
			double length = Math.sqrt(
				                	endX * endX +
				                	endY * endY
				        	);
			
			return ((Math.abs(
		    			pointX * endY - pointY * endX
			    	 ) / length) < SHIFT) 
			    	 && ((Math.abs(
			    			 pointX * endX + pointY * endY
			    	      ) < (length * length)))
			    	 && (pointX * endX + pointY * endY >= 0);
		}

		public static boolean isInEdgeLabel(Graphics gc, 
		        		IEdge edge, int x, int y) {
			int movedX = x - getLabelLeft(gc, edge);
			int movedY = y - getLabelTop(gc, edge);
			
			double angle = Math.atan2(movedY, movedX) -
	                     		getAngle(gc, edge);
			
			double length = 
				Math.sqrt(movedX * movedX + movedY * movedY);
			
			int rotatedX = (int) (length * Math.cos(angle));
			int rotatedY = (int) (length * Math.sin(angle));
			
			int width = getLabelWidth(gc, edge);
			int height = getLabelHeight(gc, edge);

			if ((rotatedX > width) 
					|| (rotatedX < 0)) {
				return false;
			}
			
			return (rotatedY > 0) && (rotatedY < height);
		}

		public static boolean compareEdges(IEdge edge1, IEdge edge2) {
			if (edge1 == null) {
				return edge2 == null;
			}
			
			return (edge2 != null) 
	        		&& edge1.getStart().equals(edge2.getStart())
	         		&& edge1.getEnd().equals(edge2.getEnd());
		}
		
		private static double getXOnElipse(int width, int height,
	          	double slope) {
			return width * height / 2 /
	    		Math.sqrt(
		     	width * width * slope * slope + 
			    	height * height
	    		);
		}

		private static int getCoordinate(int x1, int x2, double shift) {
			return (int) (x1 + Math.signum(x2 - x1) * shift);
		}

		private static double getShift(Graphics gc, IEdge edge,
			     boolean isStart, boolean isX) {
			int x1 = edge.getStart().getX();
			int y1 = edge.getStart().getY();

			int x2 = edge.getEnd().getX();
			int y2 = edge.getEnd().getY();		

			int width;
			int height;

			if (isStart) {
				width = VertexDataCalculator.getWidth(gc, edge.getStart());
				height = VertexDataCalculator.getHeight(gc, edge.getStart());
			} else {
				width = VertexDataCalculator.getWidth(gc, edge.getEnd());
				height = VertexDataCalculator.getHeight(gc, edge.getEnd());
	    	}

			if ((x2 - x1) == 0) {
	    		return isX ? 0 : height / 2;
	    	} 

			double slope = Math.abs(
				       ((double) y2 - y1) / (x2 - x1) 
				   );

			if (isX) {
				return getXOnElipse(width, height, slope);
			} 

			return slope * getXOnElipse(width, height, slope);
		}

		public static int getStartX(Graphics gc, IEdge edge) {
			return getCoordinate(
	    		edge.getStart().getX(), 
		    	edge.getEnd().getX(), 
			    getShift(gc, edge, true, true)
			);
		}

		public static int getStartY(Graphics gc, IEdge edge) {
			return getCoordinate(
	    		edge.getStart().getY(), 
		    	edge.getEnd().getY(), 
	    		getShift(gc, edge, true, false)
			);
		}

		public static int getEndX(Graphics gc, IEdge edge) {
			return getCoordinate(
	    		edge.getEnd().getX(),
		    	edge.getStart().getX(), 
	    		getShift(gc, edge, false, true)
			);
		}

		public static int getEndY(Graphics gc, IEdge edge) {
			return getCoordinate(
	    		edge.getEnd().getY(), 
		    	edge.getStart().getY(), 
	    		getShift(gc, edge, false, false)
			);
		}

		public static double getLength(Graphics gc, IEdge edge){
			int x1 = getStartX(gc, edge);
			int y1 = getStartY(gc, edge);
			int x2 = getEndX(gc, edge);
			int y2 = getEndY(gc, edge);

			return Math.sqrt((double)(x2 - x1) * 
					(x2 - x1) + (y2 - y1) * (y2 - y1));
		}
		
		
		public static double getMiddleX(IEdge edge) {
			return edge.getStart().getX() +
	    		(double) (edge.getEnd().getX() - 
	    				edge.getStart().getX()) / 2;
		}

		public static double getMiddleY(IEdge edge) {
			return edge.getStart().getY() +
	    		(double) (edge.getEnd().getY() - 
	    				edge.getStart().getY()) / 2;
		}

		public static double getAngle(Graphics gc, IEdge edge) {
			return Math.atan2(
					edge.getEnd().getY() - edge.getStart().getY(), 
					edge.getEnd().getX() - edge.getStart().getX()
				   );
		}
		
		public static int getLabelWidth(Graphics gc, IEdge edge) {
			return gc.getFontMetrics().stringWidth(edge.getLabel());
		}
		
		public static int getLabelHeight(Graphics gc, IEdge edge) {
			return gc.getFontMetrics().getHeight();
		}

		public static int getLabelLeft(Graphics gc, IEdge edge) {
			return getStartX(gc, edge) + 
	        		(getEndX(gc, edge) - getStartX(gc, edge)) / 2 -
	            		(int) (getLabelWidth(gc, edge) * 
	    	                 Math.cos(getAngle(gc, edge)) / 2 -
	    	                 getLabelHeight(gc, edge) *
	    	                 Math.sin(getAngle(gc, edge)));
		}
		
		public static int getLabelTop(Graphics gc, IEdge edge) {
			return getStartY(gc, edge) + 
	        		(getEndY(gc, edge) - getStartY(gc, edge)) / 2 - 
	            		(int) (getLabelHeight(gc, edge) *
	                 		 Math.cos(getAngle(gc, edge)) +
	                		 getLabelWidth(gc, edge) / 2 *
	                		 Math.sin(getAngle(gc, edge)));
		}
	}
