package ru.amse.smyshlyaev.grapheditor.ui.drawer;

import java.awt.*;
import java.awt.font.*;

import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.graphdatacalculator.*;


public class EdgeDrawer {
	private IEdge myEdge;
	private static final int MY_ARROW_LENGTH = 10; 
	private static final double MY_ARROW_ANGLE = Math.PI / 14;
	
	public EdgeDrawer(IEdge edge) {
		myEdge = edge;
	}

	public EdgeDrawer() {
		this(null);
	}

	public void setEdge(IEdge edge) {
		myEdge = edge;
	}
	
	private void drawArrowHalf(Graphics gc, int signum) {
		double angle = 
			EdgeDataCalculator.getAngle(gc, myEdge);
		
		int x = (int) (MY_ARROW_LENGTH *
	                      Math.cos(
	                          MY_ARROW_ANGLE - signum * angle
	                      )
	                  );
	
    	int y = (int) (MY_ARROW_LENGTH *
			              Math.sin(
			    	          MY_ARROW_ANGLE - signum * angle
			              )
			          );
		
    	int endX = EdgeDataCalculator.getEndX(gc, myEdge);
    	int endY = EdgeDataCalculator.getEndY(gc, myEdge);

    	gc.drawLine(endX, endY, endX - x, endY + signum * y);		
	}
	
	private void drawArrow(Graphics gc) {
		if (EdgeDataCalculator.getLength(gc, myEdge) < 
				(MY_ARROW_LENGTH * 
						Math.cos(MY_ARROW_ANGLE))) {
			return;
		}
		
		drawArrowHalf(gc, 1);
		drawArrowHalf(gc, -1);
	}
	
	private void drawLine(Graphics gc) {
		gc.drawLine(
				EdgeDataCalculator.getStartX(gc, myEdge),
				EdgeDataCalculator.getStartY(gc, myEdge),
				EdgeDataCalculator.getEndX(gc, myEdge),
				EdgeDataCalculator.getEndY(gc, myEdge)
		);
	}
	
	private void drawText(Graphics gc) {
		if (myEdge.getLabel().length() == 0) {
			return;
		}

		if (EdgeDataCalculator.getLabelWidth(gc, myEdge) > 
        		EdgeDataCalculator.getLength(gc, myEdge)) {
			return;
		}
		
		Graphics2D g2d = (Graphics2D) gc;

		g2d.translate(
				EdgeDataCalculator.getMiddleX(myEdge),
				EdgeDataCalculator.getMiddleY(myEdge)
		);
		
		g2d.rotate(EdgeDataCalculator.getAngle(g2d, myEdge));
		
		final int SPACE = 3;
		
		TextLayout tl = new TextLayout(
		    	myEdge.getLabel(),
		    	g2d.getFont(),
	        	g2d.getFontRenderContext()
	    );

		tl.draw(
				g2d,
				-EdgeDataCalculator.getLabelWidth(
						g2d, 
						myEdge
				) / 2,
				-SPACE
		);
		
		g2d.rotate(-EdgeDataCalculator.getAngle(g2d, myEdge));		
		g2d.translate(
				-EdgeDataCalculator.getMiddleX(myEdge),
				-EdgeDataCalculator.getMiddleY(myEdge)
		);
	}
	
	public void draw(Graphics gc) {
		if ((myEdge == null)
				|| (myEdge.getStart() == null)
				|| (myEdge.getEnd() == null)
				|| (VertexDataCalculator.isIntersection(
			           gc, 
			           myEdge.getStart(),
        			   myEdge.getEnd()
	        	   ))) {
			return;
		}
		
		drawLine(gc);
		drawText(gc);
		
        drawArrow(gc);
	}		
}