package ru.amse.smyshlyaev.grapheditor.ui.tool.labeleditor;

import java.awt.Rectangle;
import java.awt.event.*;
import javax.swing.*;
import java.awt.Graphics;

import ru.amse.smyshlyaev.grapheditor.ui.JGraphComponent;
import ru.amse.smyshlyaev.grapheditor.ui.tool.*;

/*package*/ abstract class LabelEditor extends ToolAdapter
                            implements ILabelEditor {
	protected final JGraphComponent myComponent;
	protected JTextField myLabelTextField;
	private boolean shouldExit;
	
	public LabelEditor(JGraphComponent component) {
		myComponent = component;
		
		myLabelTextField = new JTextField();
		myLabelTextField.setHorizontalAlignment(
				JTextField.LEFT
		);

		myComponent.add(myLabelTextField);
		myLabelTextField.addKeyListener(getKeyListener());

		myLabelTextField.setVisible(false);
		
		shouldExit = false;
	}

	abstract protected int getRectangleX();
	
	abstract protected int getRectangleY();

	protected int getRectangleWidth(Graphics gc) {
		return myLabelTextField.getFontMetrics(
    				   myLabelTextField.getFont()
	           ).stringWidth(myLabelTextField.getText()) + 10;
	}
	
	protected int getRectangleHeight(Graphics gc) {
		return myLabelTextField.getFontMetrics(
	    	           myLabelTextField.getFont()
        	   ).getHeight() + 4;
	}
	
	public void paint(Graphics gc) {
		if (!myLabelTextField.isVisible()) {
	     	myLabelTextField.setText(getOldLabel());				
		}

		myLabelTextField.setLocation(
				getRectangleX(),
				getRectangleY()
		);
		
		myLabelTextField.setSize(
		    	getRectangleWidth(gc),
		    	getRectangleHeight(gc)
		);
		
		myLabelTextField.setVisible(true);
		myLabelTextField.requestFocusInWindow();
		
		myLabelTextField.repaint();
		
	}	

	public void exitTextField(String newLabel) {
        myLabelTextField.setVisible(false);
		myComponent.setDefaultTool();
	}	
	
	abstract protected String getOldLabel();
	
	protected String getNewLabel() {
		return myLabelTextField.getText();
	}
	
	private KeyListener getKeyListener() {
		return new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				switch (e.getKeyCode()) {
				case KeyEvent.VK_ESCAPE:
					exitTextField(getOldLabel());
					break;
				case KeyEvent.VK_ENTER:
					exitTextField(getNewLabel());
					break;
				default:
					myComponent.repaint();
				}
			}
		};
	}

	public void mousePressed(MouseEvent e) {
     	Rectangle rectangle = new Rectangle(
        		(int) myLabelTextField.getLocation().getX(),
        		(int) myLabelTextField.getLocation().getY(),
        		myLabelTextField.getWidth(),
       			myLabelTextField.getHeight()
        );
		
		if (!rectangle.contains(e.getX(), e.getY())) {
			shouldExit = true;
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		if (shouldExit) {
			shouldExit = false;
			exitTextField(getNewLabel());
		}
	}
}