package ru.amse.smyshlyaev.grapheditor.ui.tool.vertexcreator;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import ru.amse.smyshlyaev.grapheditor.ui.tool.*;
import ru.amse.smyshlyaev.grapheditor.ui.*;
import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.graphdatacalculator.*;
import ru.amse.smyshlyaev.grapheditor.ui.drawer.*;


public class VertexCreator extends ToolAdapter 
                           implements ITool {
	private final JGraphComponent myComponent;
	protected ISelectableVertex myVertex;
	private VertexDrawer myVertexDrawer;
	private boolean shouldPaint;
	private static final int myTransparencyValue = 100;
	private static final Color MY_GREEN = 
                    		new Color(0, 255, 0, myTransparencyValue); 
	private static final Color MY_RED = 
                     		new Color(255, 0, 0, myTransparencyValue);
	
	public VertexCreator(JGraphComponent component) {
		myComponent = component;
		myVertex = new Vertex();
		myVertexDrawer = new VertexDrawer();
		shouldPaint = false;
	}
	
	private void hideVertex() {
		shouldPaint = false;
		myComponent.repaint();		
	}
	
	public void paint(Graphics gc) {
		if (!shouldPaint) {
			return;
		}

		myVertexDrawer.setVertex(myVertex);
		myVertexDrawer.draw(gc);		
	}

	public void mouseExited(MouseEvent e) {
		hideVertex();
	}

	protected void initVertex() {
		myVertex = new Vertex();
	}
	
	public void mousePressed(MouseEvent e) {
		if (myVertexDrawer.getColor() == MY_GREEN) {
			myVertex.setLocation(
					e.getX(),
					e.getY()
			);

			myComponent.addVertex(myVertex);
			initVertex();
			
			hideVertex();
		}
	}

	public void mouseMoved(MouseEvent e) {
		shouldPaint = true;
		
		myVertex.setLocation(
				e.getX(),
				e.getY()
		);

		Set<ISelectableVertex> vertices = 
	     		myComponent.getGraph().getVertices();
		
		for (ISelectableVertex v : vertices) {
			if (VertexDataCalculator.isIntersection(
					myComponent.getGraphics(), 
					v, 
					myVertex
				)) {
				myVertexDrawer.setColor(MY_RED);
				myComponent.repaint();
				return;
			}
		}

		myVertexDrawer.setColor(MY_GREEN);
		myComponent.repaint();
	}
}